"""
Module for Terraform Cloud API Endpoint: Registry Providers.
"""

from .endpoint import TFCEndpoint

class TFCRegistryProviders(TFCEndpoint):
    """
    `Registry Providers API Docs \
        <https://www.terraform.io/docs/cloud/api/providers.html>`_
    """
    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._org_providers_base_url = f"{self._api_v2_base_url}/organizations/{self._org_name}/registry-providers"

    def required_entitlements(self):
        return []

    def terraform_cloud_only(self):
        return False

    def terraform_enterprise_only(self):
        return False

    def create(self, payload):
        """
        ``POST /organizations/:organization_name/registry-providers``

        `Registry Providers Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/providers.html#create-a-provider>`_

        `Registry Providers Create Sample Payload \
            <https://www.terraform.io/docs/cloud/api/providers.html#sample-payload-public-provider->`_
        """
        return self._create(self._org_providers_base_url, payload)


    def list(self, page=None, page_size=None, filters=None):
        """
        ``GET /organizations/:organization_name/registry-providers``

        `Registry Providers List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/providers.html#list-public-terraform-registry-providers-for-an-organization>`_

        `Query Parameter(s) Details \
            <https://www.terraform.io/docs/cloud/api/providers.html#query-parameters>`__
        """
        return self._list(self._org_providers_base_url, page=page, page_size=page_size, filters=filters)

    def list_all(self):
        """
        This function does not correlate to an endpoint in the TFC API Docs specifically,
        but rather is a helper function to wrap the `list` endpoint, which enumerates out
        every page so users do not have to implement the paging logic every time they just
        want to list every run trigger for a workspace.

        Returns an object with two arrays of objects.
        """
        return self._list_all(self._org_providers_base_url)

    def show(self, registry_name, namespace, name):
        """
        ``GET /organizations/:organization_name/registry-providers/:registry_name/:namespace/:name``

        `Registry Providers Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/providers.html#get-a-provider>`_
        """
        url = f"{self._org_providers_base_url}/{registry_name}/{namespace}/{name}"
        return self._show(url)

    def destroy(self, registry_name, namespace, name):
        """
        ``DELETE /organizations/:organization_name/registry-providers/:registry_name/:namespace/:name``

        `Registry Providers Destroy API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/providers.html#delete-a-provider>`_
        """
        url = f"{self._org_providers_base_url}/{registry_name}/{namespace}/{name}"
        return self._destroy(url)



"""
Module for Terraform Cloud API Endpoint: Variable Sets.
"""

from .endpoint import TFCEndpoint


class TFCVarSets(TFCEndpoint):
    """
    `Variable Sets API Docs \
        <https://www.terraform.io/docs/cloud/api/variable-sets.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._org_varsets_base_url = f"{self._api_v2_base_url}/organizations/{self._org_name}/varsets"
        self._endpoint_base_url = f"{self._api_v2_base_url}/varsets"
        self._ws_api_v2_base_url = f"{self._api_v2_base_url}/workspaces"

    def required_entitlements(self):
        return []

    def terraform_cloud_only(self):
        # TODO: change when it gets released to TFE
        return True

    def terraform_enterprise_only(self):
        return False

    def create(self, payload):
        """
        ``POST organizations/:organization_name/varsets``

        `Variable Sets Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#create-a-variable-set>`_

        `Create Sample Payload \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#sample-payload>`_
        """
        return self._create(self._org_varsets_base_url, payload)

    def show(self, varset_id):
        """
        ``GET varsets/:varset_id``

        `Variable Sets Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#show-variable-set>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}"
        return self._show(url)

    def list_for_org(self):
        """
        ``GET organizations/:organization_name/varsets``

        `Variable Sets List For Organization API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#list-variable-set>`_
        """
        return self._list(self._org_varsets_base_url)

    # FIXME: list_all_for_org when pagination is supported

    def list_for_workspace(self, workspace_id):
        """
        ``GET workspaces/:workspace_id/varsets``

        `Variable Sets List For Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#list-variable-set>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/varsets"
        return self._list(url)

    # FIXME: list_all_for_workspace when pagination is supported

    def update(self, varset_id, payload):
        """
        ``PUT/PATCH varsets/:varset_id``

        `Variable Sets Update API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#update-a-variable-set>`_

        `Update Sample Payload \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#sample-payload-1>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}"
        return self._update(url, payload)

    def destroy(self, varset_id):
        """
        ``DELETE varsets/:varset_id``

        `Variable Sets Destroy API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#delete-a-variable-set>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}"
        return self._destroy(url)

    # VARIABLE RELATIONSHIPS
    def add_var_to_varset(self, varset_id, payload):
        """
        ``POST varsets/:varset_external_id/relationships/vars``

        `Variable Sets Add Variable API Doc Reference \
            <POST varsets/:varset_external_id/relationships/vars>`_

        `Add Variable To Variable Set Sample Payload \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#sample-payload-2>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}/relationships/vars"
        return self._post(url, data=payload)

    def update_var_in_varset(self, varset_id, var_id, payload):
        """
        ``PATCH varsets/:varset_id/relationships/vars/:var_id``

        `Variable Sets Update Variable API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#update-a-variable-in-a-variable-set>`_

        `Update Variable In Variable Set Sample Payload \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#sample-payload-3>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}/relationships/vars/{var_id}"
        return self._update(url, payload)

    def delete_var_from_varset(self, varset_id, var_id):
        """
        ``DELETEvarsets/:varset_id/relationships/vars/:var_id``

        `Variable Sets Delete Variable API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#delete-a-variable-in-a-variable-set>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}/relationships/vars/{var_id}"
        return self._destroy(url)

    def list_vars_in_varset(self, varset_id):
        """
        ``GET varsets/:varset_id/relationships/vars``

        `Variable Sets List In Variable Set API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#list-variables-in-a-variable-set>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}/relationships/vars"
        return self._list(url)

    # WORKSPACE RELATIONSHIPS
    def apply_varset_to_workspace(self, varset_id, payload):
        """
        ``POST varsets/:varset_id/relationships/workspaces``

        `Variable Sets Apply To Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#apply-variable-set-to-workspaces>`_

        `Apply Variable Set to Workspace Sample Payload \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#sample-payload-4>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}/relationships/workspaces"
        return self._post(url, data=payload)

    def remove_varset_from_workspace(self, varset_id, payload):
        """
        ``DELETE varsets/:varset_id/relationships/workspaces``

        `Variable Sets Remove From Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#remove-a-variable-set-from-workspaces>`_

        `Remove Variable Set From Workspace Sample Payload \
            <https://www.terraform.io/docs/cloud/api/variable-sets.html#sample-payload-5>`_
        """
        url = f"{self._endpoint_base_url}/{varset_id}/relationships/workspaces"
        self._destroy(url, data=payload)



"""
Module for Terraform Cloud API Endpoint: Admin Terraform Versions.
"""

from .endpoint import TFCEndpoint

class TFCAdminTerraformVersions(TFCEndpoint):
    """
    `Admin Terraform Versions API Docs \
        <https://www.terraform.io/docs/cloud/api/admin/terraform-versions.html>`_
    """

    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._endpoint_base_url = f"{self._api_v2_base_url}/admin/terraform-versions"

    def list(self, page=None, page_size=None):
        """
        ``GET /api/v2/admin/terraform-versions``

        `Admin Terraform Versions List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/terraform-versions.html#list-all-terraform-versions>`_
        """
        return self._list(self._endpoint_base_url, page=page, page_size=page_size)

    def list_all(self):
        """
        This function does not correlate to an endpoint in the TFC API Docs specifically,
        but rather is a helper function to wrap the `list` endpoint, which enumerates out
        every page so users do not have to implement the paging logic every time they just
        want to list every workspace in an organization.

        Returns an object with two arrays of objects.
        """
        return self._list_all(self._endpoint_base_url)

    def required_entitlements(self):
        return []

    def terraform_cloud_only(self):
        return False

    def terraform_enterprise_only(self):
        return True

    def create(self, data):
        """
        ``POST /admin/terraform-versions``

        `Admin Terraform Versions Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/terraform-versions.html#create-a-terraform-version>`_
        """
        return self._post(self._endpoint_base_url, data=data)

    def show(self, version_id):
        """
        ``GET /api/v2/admin/terraform-versions/:id``

        `Admin Terraform Versions Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/terraform-versions.html#show-a-terraform-version>`_
        """
        url = f"{self._endpoint_base_url}/{version_id}"
        return self._show(url)

    def update(self, version_id, data):
        """
        ``PATCH /admin/terraform-versions/:id``

        `Admin Terraform Versions Update API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/terraform-versions.html#update-a-terraform-version>`_
        """
        url = f"{self._endpoint_base_url}/{version_id}"
        return self._patch(url, data)

    def destroy(self, version_id):
        """
        ``DELETE /admin/terraform-versions/:id``

        `Admin Terraform Versions Destroy API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/admin/terraform-versions.html#delete-a-terraform-version>`_
        """
        url = f"{self._endpoint_base_url}/{version_id}"
        return self._destroy(url)



"""
Module for Terraform Cloud API Endpoint: Run Tasks.
"""

from .endpoint import TFCEndpoint

class TFCRunTasks(TFCEndpoint):
    """
    `Run Tasks API Docs \
        <https://www.terraform.io/docs/cloud/api/run-tasks.html>`_
    """
    def __init__(self, instance_url, org_name, headers, well_known_paths, verify, log_level):
        super().__init__(instance_url, org_name, headers, well_known_paths, verify, log_level)
        self._org_tasks_base_url = f"{self._api_v2_base_url}/organizations/{self._org_name}/tasks"
        self._tasks_base_url = f"{self._api_v2_base_url}/tasks"
        self._ws_api_v2_base_url = f"{self._api_v2_base_url}/workspaces"

    def required_entitlements(self):
        return []

    def terraform_cloud_only(self):
        # FIXME: remove this once it goes to TFE GA
        return True

    def terraform_enterprise_only(self):
        return False

    def create(self, payload):
        """
        ``POST /organizations/:organization_name/tasks``

        `Run Tasks Create API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#create-a-run-task>`_

        `Run Tasks Create Sample Payload \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#sample-payload>`_
        """
        return self._create(self._org_tasks_base_url, payload)


    def list(self, page=None, page_size=None, include=None):
        """
        ``GET /organizations/:organization_name/tasks``

        `Run Tasks List API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#list-run-tasks>`_

        `Query Parameter(s) Details \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#query-parameters>`__
        """
        return self._list(self._org_tasks_base_url, page=page, page_size=page_size, include=include)

    def list_all(self):
        """
        This function does not correlate to an endpoint in the TFC API Docs specifically,
        but rather is a helper function to wrap the `list` endpoint, which enumerates out
        every page so users do not have to implement the paging logic every time they just
        want to list every run trigger for a workspace.

        Returns an object with two arrays of objects.
        """
        return self._list_all(self._org_tasks_base_url)

    def show(self, task_id, include=None):
        """
        ``GET /tasks/:id``

        `Run Tasks Show API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#show-a-run-task>`_
        """
        url = f"{self._tasks_base_url}/{task_id}"
        return self._show(url, include=include)

    def update(self, task_id, payload):
        """
        ``PATCH /tasks/:id``

        `Run Tasks Update API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#update-a-task>`_

        `Update Sample Payload \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#sample-payload-1>`_
        """
        url = f"{self._tasks_base_url}/{task_id}"
        return self._update(url, payload)

    def destroy(self, task_id):
        """
        ``DELETE /tasks/:id``

        `Run Tasks Destroy API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#delete-a-run-task>`_
        """
        url = f"{self._tasks_base_url}/{task_id}"
        return self._destroy(url)

    # WORKSPACE RELATIONSHIPS
    def attach_task_to_workspace(self, workspace_id, payload):
        """
        ``POST /workspaces/:workspace_id/tasks``

        `Run Tasks Attach To Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#attach-a-run-task-to-a-workspace>`_

        `Attach Sample Payload \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#sample-payload-2>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/tasks"
        return self._post(url, data=payload)

    def list_tasks_on_workspace(self, workspace_id, page=None, page_size=None):
        """
        ``GET /workspaces/:workspace_id/tasks``

        `Run Tasks List On Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#list-workspace-run-tasks>`_

        `Query Parameter(s) Details \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#query-parameters-1>`__
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/tasks"
        return self._get(url, page=page, page_size=page_size)

    def list_all_tasks_on_workspace(self, workspace_id):
        """
        This function does not correlate to an endpoint in the TFC API Docs specifically,
        but rather is a helper function to wrap the `list` endpoint, which enumerates out
        every page so users do not have to implement the paging logic every time they just
        want to list every policy for an organization.

        Returns an object with two arrays of objects.
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/tasks"
        return self._list_all(url)

    def show_task_on_workspace(self, workspace_id, task_id):
        """
        ``GET /workspaces/:workspace_id/tasks/:id``

        `Run Tasks Show On Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#show-workspace-run-task>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/tasks/{task_id}"
        return self._get(url)

    def update_task_on_workspace(self, workspace_id, task_id, payload):
        """
        ``PATCH /workspaces/:workspace_id/tasks/:id``

        `Run Tasks Update On Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#update-workspace-run-task>`_

        `Update On Workspace Sample Payload \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#sample-payload-3>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/tasks/{task_id}"
        return self._update(url, payload=payload)

    def remove_task_from_workspace(self, workspace_id, task_id):
        """
        ``DELETE /workspaces/:workspace_id/tasks/:id``

        `Run Tasks Remove From Workspace API Doc Reference \
            <https://www.terraform.io/docs/cloud/api/run-tasks.html#delete-workspace-task>`_
        """
        url = f"{self._ws_api_v2_base_url}/{workspace_id}/tasks/{task_id}"
        return self._destroy(url)



"""
Module for testing the Terraform Cloud API Endpoint: Registry Providers.
"""

from .base import TestTFCBaseTestCase


class TestTFCRegistryProviders(TestTFCBaseTestCase):
    """
    Class for testing the Terraform Cloud API Endpoint: Registry Providers.
    """

    _unittest_name = "prov"
    _endpoint_being_tested = "registry_providers"

    def test_providers(self):
        """
        Test the Registry Providers API endpoints.
        """

        # List all the providers in the org, confirm there are none
        listed_providers = self._api.registry_providers.list()["data"]
        self.assertEqual(len(listed_providers), 0)

        # Create a provider, confirm there is one provider
        create_payload = {
            "data": {
                "type": "registry-providers",
                "attributes": {
                    "name": "aws",
                    "namespace": "hashicorp",
                    "registry-name": "public"
                }
            }
        }
        created_provider = self._api.registry_providers.create(create_payload)["data"]
        created_provider_id = created_provider["id"]
        created_provider_registry_name = created_provider["attributes"]["registry-name"]
        created_provider_namespace = created_provider["attributes"]["namespace"]
        created_provider_name = created_provider["attributes"]["name"]

        # Confirm there is one provider after creating one
        listed_providers = self._api.registry_providers.list()["data"]
        self.assertEqual(len(listed_providers), 1)

        # Confirm there is one provider after creating one with list all
        all_listed_providers = self._api.registry_providers.list_all()["data"]
        self.assertEqual(len(all_listed_providers), 1)

        # Show the provider
        shown_provider = self._api.registry_providers.show(\
            created_provider_registry_name, created_provider_namespace, created_provider_name)["data"]
        # Confirm the shown provider matches the created provider ID
        self.assertEqual(shown_provider["id"], created_provider_id)

        # Destroy the created provider
        self._api.registry_providers.destroy(\
            created_provider_registry_name, created_provider_namespace, created_provider_name)

        # List the providers and confirm there are none
        listed_providers = self._api.registry_providers.list()["data"]
        self.assertEqual(len(listed_providers), 0)
